local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local BlizzardUIModule = idTweaks:NewModule('BlizzardUI', 'AceConsole-3.0', 'AceHook-3.0')

BlizzardUIModule.DEBUG = false

function BlizzardUIModule:Debug(...)
  if BlizzardUIModule.DEBUG then
    self:Print(...)
  end
end

function BlizzardUIModule:setDefaults()
  self:Debug('BlizzardUIModule:setDefaults')

  self.defaults = {
    profile = {
      enabled = true,

      buffFrame = {
        hide = true,
      },

      castingBarFrame = {
        hide = true,
      },

      errorsFrame = {
        abilityIsNotReadyYet = true,
        outOfRange = true,
        spellIsNotReadyYet = true,
        targetNeedsToBeInFrontOfYou = true,
        targetTooClose = true,
        thereIsNothingToAttack = true,
        youAreFacingTheWrongWay = true,
        youAreTooFarAway = true,
        youCantDoThatYet = true,
        youHaveNoTarget = true,
      }
    }
  }
end

function BlizzardUIModule:setDatabase()
  self:Debug('BlizzardUIModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('BlizzardUI', self.defaults)
end

function BlizzardUIModule:setOptions()
  self:Debug('BlizzardUIModule:setOptions')

  self.options = {
    name = 'BlizzardUI',
    handler = self,
    type = 'group',
    args = {
      enabled = {
        order = 1,
        name = 'Enabled',
        desc = 'Enables / Disables the Module',
        handler = self,
        type = 'toggle',
        get = function()
          return self.db.profile.enabled
        end,
        set = function(_, value)
          self.db.profile.enabled = value
          self:check()
        end,
      },

      buffFrame = {
        name = 'Buffs',
        type = 'group',
        disabled = function()
          return not self.db.profile.enabled
        end,
        args = {
          hide = {
            name = 'Hide',
            type = 'toggle',
            get = function()
              return self.db.profile.buffFrame.hide
            end,
            set = function(_, value)
              self.db.profile.buffFrame.hide = value
              self.buffFrame:check()
            end
          },
        }
      },

      castingBarFrame = {
        name = 'Casting Bar',
        type = 'group',
        disabled = function()
          return not self.db.profile.enabled
        end,
        args = {
          hide = {
            name = 'Hide',
            type = 'toggle',
            get = function()
              return self.db.profile.castingBarFrame.hide
            end,
            set = function(_, value)
              self.db.profile.castingBarFrame.hide = value
              self.castingBarFrame:check()
            end
          },
        }
      },

      errorsFrame = {
        name = 'Errors',
        type = 'group',
        disabled = function()
          return not self.db.profile.enabled
        end,
        args = {
          abilityIsNotReadyYet = {
            name = 'Ability is not ready yet.',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.abilityIsNotReadyYet
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.abilityIsNotReadyYet = value
              self.errorsFrame:check()
            end
          },

          outOfRange = {
            name = 'Out of range.',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.outOfRange
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.outOfRange = value
              self.errorsFrame:check()
            end
          },

          spellIsNotReadyYet = {
            name = 'Spell is not ready yet.',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.spellIsNotReadyYet
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.spellIsNotReadyYet = value
              self.errorsFrame:check()
            end
          },

          targetNeedsToBeInFrontOfYou = {
            name = 'Target needs to be in front of you.',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.targetNeedsToBeInFrontOfYou
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.targetNeedsToBeInFrontOfYou = value
            end
          },

          targetTooClose = {
            name = 'Target too close',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.targetTooClose
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.targetTooClose = value
            end
          },

          thereIsNothingToAttack = {
            name = 'There is nothing to attack.',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.thereIsNothingToAttack
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.thereIsNothingToAttack = value
            end
          },

          youAreFacingTheWrongWay = {
            name = 'You are facing the wrong way!',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.youAreFacingTheWrongWay
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.youAreFacingTheWrongWay = value
            end
          },

          youAreTooFarAway = {
            name = 'You are too far away!',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.youAreTooFarAway
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.youAreTooFarAway = value
            end
          },

          youCantDoThatYet = {
            name = 'You can\'t do that yet',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.youCantDoThatYet
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.youCantDoThatYet = value
            end
          },

          youHaveNoTarget = {
            name = 'You have no target.',
            type = 'toggle',
            get = function()
              return self.db.profile.errorsFrame.youHaveNoTarget
            end,
            set = function(_, value)
              self.db.profile.errorsFrame.youHaveNoTarget = value
            end
          },
        }
      }
    }
  }
end

function BlizzardUIModule:registerOptions()
  self:Debug('BlizzardUIModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('idTweaks-BlizzardUI', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('idTweaks-BlizzardUI', 'BlizzardUI', 'idTweaks')
end

function BlizzardUIModule:OnInitialize()
  self:Debug('BlizzardUIModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function BlizzardUIModule:OnEnable()
  self:Debug('BlizzardUIModule:OnEnable')

  self:registerOptions()

  self:check()
end

function BlizzardUIModule:OnDisable()
  self:Debug('BlizzardUIModule:OnDisable')

  self:disable()
end

function BlizzardUIModule:check()
  self:Debug('BlizzardUIModule:check')

  if self.db.profile.enabled then
    self:enable()
  else
    self:disable()
  end
end

function BlizzardUIModule:enable()
  self:Debug('BlizzardUIModule:enable')

  self.buffFrame:check()
  self.castingBarFrame:check()
  -- self.errorsFrame:check()
end

function BlizzardUIModule:disable()
  self:Debug('BlizzardUIModule:disable')

  self.buffFrame:disable()
  self.castingBarFrame:disable()
  -- self.errorsFrame:disable()
end

BlizzardUIModule.buffFrame = {}

function BlizzardUIModule.buffFrame:check()
  BlizzardUIModule:Debug('BlizzardUIModule.buffFrame:checkEnabled')

  if BlizzardUIModule.db.profile.buffFrame.hide then
    BlizzardUIModule.buffFrame:enable()
  else
    BlizzardUIModule.buffFrame:disable()
  end
end

function BlizzardUIModule.buffFrame:enable()
  BlizzardUIModule:Debug('BlizzardUIModule.buffFrame:enable')

  BlizzardUIModule:SecureHook(BuffFrame, 'Show', function(frame)
    if (BlizzardUIModule.db.profile.buffFrame.hide) then
      BuffFrame:Hide()
    end
  end)
  if (BlizzardUIModule.db.profile.buffFrame.hide) then
    BuffFrame:Hide()
  end
end

function BlizzardUIModule.buffFrame:disable()
  BlizzardUIModule:Debug('BlizzardUIModule.buffFrame:disable')

  BlizzardUIModule:Unhook(BuffFrame, 'Show')

  BuffFrame:Show()
end

BlizzardUIModule.castingBarFrame = {}

function BlizzardUIModule.castingBarFrame:check()
  BlizzardUIModule:Debug('BlizzardUIModule.castingBarFrame:checkEnabled')

  if BlizzardUIModule.db.profile.castingBarFrame.hide then
    BlizzardUIModule.castingBarFrame:enable()
  else
    BlizzardUIModule.castingBarFrame:disable()
  end
end

function BlizzardUIModule.castingBarFrame:enable()
  BlizzardUIModule:Debug('BlizzardUIModule.castingBarFrame:enable')

  BlizzardUIModule:SecureHook(CastingBarFrame, 'Show', function(frame)
    if (BlizzardUIModule.db.profile.castingBarFrame.hide) then
      CastingBarFrame:Hide()
    end
  end)
end

function BlizzardUIModule.castingBarFrame:disable()
  BlizzardUIModule:Debug('BlizzardUIModule.castingBarFrame:disable')

  BlizzardUIModule:Unhook(CastingBarFrame, 'Show')
end

BlizzardUIModule.errorsFrame = {}

BlizzardUIModule:RawHook(UIErrorsFrame, 'AddMessage', function(frame, message, red, green, blue, messageId, holdTime)
  if not BlizzardUIModule.db.profile.enabled then
    BlizzardUIModule.hooks[frame].AddMessage(frame, message, red, green, blue, messageId, holdTime)
    return
  end

  for k, v in pairs(BlizzardUIModule.db.profile.errorsFrame) do
    local option = BlizzardUIModule.options.args.errorsFrame.args[k]

    if v and option and message == option.name then
      return
    end
  end

  BlizzardUIModule.hooks[frame].AddMessage(frame, message, red, green, blue, messageId, holdTime)
end, true)

-- function BlizzardUIModule.errorsFrame:check()
--   BlizzardUIModule:Debug('BlizzardUIModule.errorsFrame:checkEnabled')
-- end
--
-- function BlizzardUIModule.errorsFrame:enable()
--   BlizzardUIModule:Debug('BlizzardUIModule.errorsFrame:enable')
-- end
--
-- function BlizzardUIModule.errorsFrame:disable()
--   BlizzardUIModule:Debug('BlizzardUIModule.errorsFrame:disable')
-- end
