local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local AceGUI = LibStub('AceGUI-3.0')
local MinimapModule = idTweaks:NewModule('Minimap', 'AceConsole-3.0')

local TL, TC, TR = 'TOPLEFT', 'TOP', 'TOPRIGHT'
local ML, MC, MR = 'LEFT', 'CENTER', 'RIGHT'
local BL, BC, BR = 'BOTTOMLEFT', 'BOTTOM', 'BOTTOMRIGHT'

MinimapModule.DEBUG = false

function MinimapModule:Debug(...)
  if MinimapModule.DEBUG then
    self:Print(...)
  end
end

function MinimapModule:setDefaults()
  self:Debug('MinimapModule:setDefaults')

  self.defaults = {
    profile = {
      enabled = true,

      shape = {
        square = true,
        position = true,
      },

      hide = {
        areaText = true,
        calendarButton = true,
        northTag = true,
        timeDisplay = true,
        worldMapButton = true,
        zoomButtons = true,
      },

      mouse = {
        mouseWheelZooming = true,
        -- rightClickMenu = true,
      },
    }
  }
end

function MinimapModule:setDatabase()
  self:Debug('MinimapModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('Minimap', self.defaults)
end

function MinimapModule:setOptions()
  self:Debug('MinimapModule:setOptions')

  self.options = {
    name = 'Minimap',
    desc = 'Changes how the Minimap works.',
    handler = self,
    type = 'group',
    args = {
      enabled = {
        order = 1,
        name = 'Enabled',
        desc = 'Enables / Disables the Module',
        handler = self,
        type = 'toggle',
        get = function()
          return self.db.profile.enabled
        end,
        set = function(_, value)
          self.db.profile.enabled = value
          self:check()
        end,
      },

      shape = {
        name = 'Shape',
        desc = 'Shape settings.',
        handler = self,
        type = 'group',
        disabled = function()
          return not self.db.profile.enabled
        end,
        args = {
          square = {
            name = 'Square',
            desc = 'Makes the minimap square',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.shape.square
            end,
            set = function(_, value)
              self.db.profile.shape.square = value
              self.shape.square:check()
            end
          },

          position = {
            name = 'Position',
            desc = 'Move the minimap next to the action bars',
            handler = self,
            type = 'toggle',
            disabled = function()
              local ActionBarsModule = idTweaks:GetModule('ActionBars')
              return ActionBarsModule.db.profile.enabled == false
            end,
            get = function()
              return self.db.profile.shape.position
            end,
            set = function(_, value)
              self.db.profile.shape.position = value
              self.shape.position:check()
            end
          },
        },
      },

      mouse = {
        name = 'Mouse',
        desc = 'Change how the mouse interacts with the Minimap.',
        handler = self,
        type = 'group',
        disabled = function()
          return not self.db.profile.enabled
        end,
        args = {
          mouseWheelZooming = {
            name = 'Mouse Wheel',
            desc = 'Enables Zooming the minimap with the Mouse Wheel.',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.mouse.mouseWheelZooming
            end,
            set = function(_, value)
              self.db.profile.mouse.mouseWheelZooming = value
              self.mouse:checkMousewheelZoomingEnabled()
            end
          },

          -- rightClickMenu = {
          --   name = 'Right Click Menu',
          --   desc = 'Enables the right click menu.',
          --   handler = self,
          --   type = 'toggle',
          --   get = function()
          --     return self.db.profile.mouse.rightClickMenu
          --   end,
          --   set = function(_, value)
          --     self.db.profile.mouse.rightClickMenu = value
          --     self.mouse:checkRightClickMenuEnabled()
          --   end
          -- }
        }
      },

      hide = {
        name = 'Hide',
        desc = 'Toggle visibility of parts of the Minimap with these options.',
        handler = self,
        type = 'group',
        disabled = function()
          return not self.db.profile.enabled
        end,
        args = {
          areaText = {
            name = 'Area Text',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.hide.areaText
            end,
            set = function(_, value)
              self.db.profile.hide.areaText = value
              self.hide:checkFrame('areaText')
            end
          },

          calendarButton = {
            name = 'Calendar Button',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.hide.calendarButton
            end,
            set = function(_, value)
              self.db.profile.hide.calendarButton = value
              self.hide:checkFrame('calendarButton')
            end
          },

          northTag = {
            name = 'North Tag',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.hide.northTag
            end,
            set = function(_, value)
              self.db.profile.hide.northTag = value
              self.hide:checkFrame('northTag')
            end
          },

          timeDisplay = {
            name = 'Time Display',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.hide.timeDisplay
            end,
            set = function(_, value)
              self.db.profile.hide.timeDisplay = value
              self.hide:checkFrame('timeDisplay')
            end
          },

          worldMapButton = {
            name = 'World Map Button',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.hide.worldMapButton
            end,
            set = function(_, value)
              self.db.profile.hide.worldMapButton = value
              self.hide:checkFrame('worldMapButton')
            end
          },

          zoomButtons = {
            name = 'Zoom Buttons',
            handler = self,
            type = 'toggle',
            get = function()
              return self.db.profile.hide.zoomButtons
            end,
            set = function(_, value)
              self.db.profile.hide.zoomButtons = value
              self.hide:checkFrame('zoomButtons')
            end
          },
        }
      }
    }
  }
end

function MinimapModule:registerOptions()
  self:Debug('MinimapModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('idTweaks-Minimap', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('idTweaks-Minimap', 'Minimap', 'idTweaks')
end

function MinimapModule:OnInitialize()
  self:Debug('MinimapModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function MinimapModule:OnEnable()
  self:Debug('MinimapModule:OnEnable')

  self:registerOptions()

  self:check()
end

function MinimapModule:OnDisable()
  self:Debug('MinimapModule:OnDisable')

  self:disable()
end

function MinimapModule:check()
  self:Debug('MinimapModule:check')

  if self.db.profile.enabled then
    self:enable()
  else
    self:disable()
  end
end

function MinimapModule:enable()
  self:Debug('MinimapModule:enable')

  self.shape.square:check()
  self.shape.position:check()
  self.mouse:check()
  self.hide:check()
end

function MinimapModule:disable()
  self:Debug('MinimapModule:disable')

  self.shape.square:disable()
  self.shape.position:disable()
  self.mouse:disable()
  self.hide:disable()
end

MinimapModule.hide = {
  frames = {
    -- MinimapTrackingFrame
    -- MinimapMailFrame
    -- MinimapBattlefieldFrame

    areaText = {
      enable = function()
        MinimapBorderTop:Hide()
        MinimapZoneText:Hide()
        MinimapZoneTextButton:Hide()
        MinimapToggleButton:Hide()
      end,
      disable = function()
        MinimapBorderTop:Show()
        MinimapZoneText:Show()
        MinimapZoneTextButton:Show()
        MinimapToggleButton:Show()
      end,
    },

    calendarButton = {
      enable = function()
        GameTimeFrame:Hide()
      end,
      disable = function()
        GameTimeFrame:Show()
      end,
    },

    northTag = {
      enable = function()
        MinimapNorthTag:SetAlpha(0)
      end,
      disable = function()
        MinimapNorthTag:SetAlpha(1)
      end,
    },

    timeDisplay = {
      enable = function()
        TimeManagerClockButton:Hide()
      end,
      disable = function()
        TimeManagerClockButton:Show()
      end,
    },

    worldMapButton = {
      enable = function()
        MiniMapWorldMapButton:Hide()
      end,
      disable = function()
        MiniMapWorldMapButton:Show()
      end,
    },

    zoomButtons = {
      enable = function()
        MinimapZoomIn:Hide()
        MinimapZoomOut:Hide()
      end,

      disable = function()
        MinimapZoomIn:Show()
        MinimapZoomOut:Show()
      end
    }
  }
}

function MinimapModule.hide:check()
  MinimapModule:Debug('MinimapModule.hide:checkEnabled')

  for name, functions in pairs(self.frames) do
    self:checkFrame(name)
  end
end

function MinimapModule.hide:disable()
  MinimapModule:Debug('MinimapModule.hide:disable')

  for name, functions in pairs(self.frames) do
    self.frames[name].disable()
  end
end

function MinimapModule.hide:checkFrame(name)
  MinimapModule:Debug('MinimapModule.hide:checkFrame')

  if MinimapModule.db.profile.hide[name] == true then
    self.frames[name].enable()
  else
    self.frames[name].disable()
  end
end

MinimapModule.mouse = {}

function MinimapModule.mouse:check()
  MinimapModule.mouse:checkMousewheelZoomingEnabled()
  -- MinimapModule.mouse:checkRightClickMenuEnabled()
end

function MinimapModule.mouse:disable()
  MinimapModule:Debug('MinimapModule.mouse:disable')

  MinimapModule.mouse:disableMouseWheelZooming()
  -- MinimapModule.mouse:disableRightClickMenu()
end

function MinimapModule.mouse:checkMousewheelZoomingEnabled()
  MinimapModule:Debug('MinimapModule.mouse:checkMousewheelZoomingEnabled')

  if MinimapModule.db.profile.mouse.mouseWheelZooming == true then
    MinimapModule.mouse:enableMouseWheelZooming()
  else
    MinimapModule.mouse:disableMouseWheelZooming()
  end
end

function MinimapModule.mouse:enableMouseWheelZooming()
  MinimapModule:Debug('MinimapModule.mouse:enableMouseWheelZooming')

  Minimap:EnableMouseWheel(true)
  Minimap:SetScript('OnMouseWheel', MinimapModule.mouse.handleMousewheelZoom)
end

function MinimapModule.mouse:disableMouseWheelZooming()
  MinimapModule:Debug('MinimapModule.mouse:disableMouseWheelZooming')

  Minimap:EnableMouseWheel(false)
  Minimap:SetScript('OnMouseWheel', nil)
end

function MinimapModule.mouse.handleMousewheelZoom(frame, delta)
  MinimapModule:Debug('MinimapModule.mouse.handleMousewheelZoom')

  if delta > 0 and Minimap:GetZoom() < 5 then
    Minimap:SetZoom(Minimap:GetZoom() + 1)
  elseif delta < 0 and Minimap:GetZoom() > 0 then
    Minimap:SetZoom(Minimap:GetZoom() - 1)
  end
end

-- MinimapModule.mouse.dropdownItems = {
--   A = 'One',
--   B = 'Two',
--   C = 'Three',
--   D = 'Four',
-- }
--
-- local rightClickDropdown = AceGUI:Create('Dropdown')
-- rightClickDropdown:SetCallback('OnClose', function(frame)
--   MinimapModule:Print('Dropdown:OnClose')
--   AceGUI:Release(frame)
-- end)
-- rightClickDropdown:SetText('AceGUI-3.0 Example')
-- rightClickDropdown:SetLabel('LABEL?')
-- rightClickDropdown:SetList(MinimapModule.mouse.dropdownItems)
--
-- MinimapModule.mouse.rightClickDropdown = rightClickDropdown
--
-- function MinimapModule.mouse:checkRightClickMenuEnabled()
--   MinimapModule:Debug('MinimapModule.mouse:checkRightClickMenuEnabled')
--
--   if MinimapModule.db.profile.mouse.rightClickMenu == true then
--     MinimapModule.mouse:enableRightClickMenu()
--   else
--     MinimapModule.mouse:disableRightClickMenu()
--   end
-- end
--
-- function MinimapModule.mouse:enableRightClickMenu()
--   MinimapModule:Debug('MinimapModule.mouse:enableRightClickMenu')
--
--   Minimap:SetScript('OnMouseUp', function(frame, button, down)
--     if button == 'RightButton' then
--       MinimapModule.mouse.rightClickDropdown:Show()
--     else
--       Minimap_OnClick(frame, button, down)
--     end
--   end)
-- end
--
-- function MinimapModule.mouse:disableRightClickMenu()
--   MinimapModule:Debug('MinimapModule.mouse:disableRightClickMenu')
--
--   Minimap:SetScript('OnClick', nil)
-- end

-- TODO: Rewrite to the style of the rest of the modules.
MinimapModule.shape = {
  square = {
    check = function()
      MinimapModule:Debug('MinimapModule.shape.square:check')

      if MinimapModule.db.profile.shape.square == true then
        MinimapModule.shape.square:enable()
      else
        MinimapModule.shape.square:disable()
      end
    end,

    enable = function()
      MinimapModule:Debug('MinimapModule.shape.square:enable')

      Minimap:SetMaskTexture('Interface\\Buttons\\WHITE8x8')
      MinimapBorder:Hide()
    end,

    disable = function()
      MinimapModule:Debug('MinimapModule.shape.square:disable')

      Minimap:SetMaskTexture('textures\\MinimapMask')
      MinimapBorder:Show()
    end,
  },

  position = {
    check = function()
      MinimapModule:Debug('MinimapModule.shape.position:check')

      local ActionBarsModule = idTweaks:GetModule('ActionBars')

      if ActionBarsModule.db.profile.enabled then
        if MinimapModule.db.profile.shape.position == true then
          MinimapModule.shape.position:enable()
        else
          MinimapModule.shape.position:disable()
        end
      else
        MinimapModule.shape.position:disable()
      end
    end,

    enable = function()
      MinimapModule:Debug('MinimapModule.shape.position:enable')

      Minimap:ClearAllPoints()
      Minimap:SetParent(UIParent)
      Minimap:SetPoint(BL, idTweaksActionBar1, BR, 0, 0)
      Minimap:SetWidth(idTweaksActionBar1:GetHeight() * 5)
      Minimap:SetHeight(idTweaksActionBar1:GetHeight() * 5)
    end,

    disable = function()
      MinimapModule:Debug('MinimapModule.shape.position:disable')

      Minimap:ClearAllPoints()
      Minimap:SetParent(MinimapCluster)
      Minimap:SetPoint(MC, MinimapCluster, TC, 9, -92)
      Minimap:SetWidth(140)
      Minimap:SetHeight(140)
    end,
  },
}

function GetMinimapShape()
  return MinimapModule.db.profile.shape.square == true and 'SQUARE' or 'ROUND'
end
